/*******************************************************************************\
*                                                                               *
* dukaudio.c -   TrueMotion/S audio interface and playback code for Sega Saturn *
*                Copyright (c) 1994, The Duck Corp. All rights reserved.        * 
*                                                                               *
*                 Editor:       Eric Ameres                                     *
*                 Version:      .96                                             *
*                 Edited:       12/07/94                                        *
					- added 8 bit stereo support
					- fixed end of audio glitch
					- added support for non-integer frame rates
*                                                                                                                                                               *
*                 Author:       Eric Ameres                                     *
*                 Version:      .95                                             *
*                 Created:      10/94                                           *
*                                                                               *
\*******************************************************************************/
#include <sega_xpt.h>
#include <sega_dbg.h>
#include <sega_snd.h>
#include <sega_int.h>

#include "duckfile.h"
#include "duksega.h"

#if DEBUG
#define DIAGNOSTIC(x) x
#else 
#define DIAGNOSTIC(x)
#endif

extern Uint32   scsp_sound_driver;
extern Uint32   scsp_sound_driver_end;
extern Uint32   scsp_systbl;
extern Uint32   scsp_systbl_end;
extern Uint32   scsp_map;
extern Uint32   scsp_map_end;                           

#define BUFSIZE 0xB000

#define B_LOAD_MARK         31
#define M_LOAD_MARK        (0x1  << B_LOAD_MARK)
#define POKE_L(adr, data)   (*((volatile Uint32 *)(adr)) = ((Uint32)(data)))

int playing = FALSE,playAudio = TRUE;
volatile long frameNum = 0,*fPtr = &frameNum;
    
unsigned long targetSample,totalSamples,targetFrameSample;
int GlobalFrameRate,GlobalBPF;
int NumChannels,SamplesPerSec,BytesPerSec,wFormat;

static short samplesPerFPS[] = {
	0,
	22050,11025,7350,5513,4410,
	3675, 3150, 2756,2450,2205,
	2005, 1838, 1696,1575,1470,
	1378, 1297, 1225,1161,1102,
	1050, 1002,  959, 919,882,
	848,   817,  788, 760,735 };

static unsigned char modFPS[] = {
	0,
	0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,2,
	0,0,0,0,0,0,0,0,0,0
	};

int aChunkIndex;                                     
static int lastpos = 0;           
	
extern DUCK_HANDLE duck_handle;
                                  
loadPCMData(register Uint8 *data,int length,int format)
{	
	int j;
	register int numToGet;
	
	j = lastpos % BUFSIZE; 
	lastpos += length;
	
	if (format < 2){ /* The 8 bit version */
		register Uint8 *outData8Left = (Uint8 *)0x25a10000;
		register Uint8 *outData8Right = (Uint8 *)(0x25a10000 + BUFSIZE);
        
		outData8Left += j;	 
		outData8Right += j;
	
		do{
			if (length + j > BUFSIZE)
				numToGet = BUFSIZE - j;
			else
				numToGet = length;			
			
			length -= numToGet;	
	        
			switch(format){
				case 0:	/* 8 bit mono to 8 bit mono */
					if (data != NULL)
						while(numToGet--) *outData8Left++ = *data++ - 128;
					else
						while(numToGet--) *outData8Left++  = 0;						
					break;
				case 1: /* 8 bit stereo to 8 bit stereo */
					if (data != NULL){
						while(numToGet--){      
							*outData8Left++ = *data++ - 128;
							*outData8Right++ = *data++ - 128;
						}
					}else
						while(numToGet--) *outData8Right = *outData8Left++  = 0;						
					break;
			}
			outData8Left = (Uint8 *)0x25a10000; outData8Right = (Uint8 *)(0x25a10000 + BUFSIZE);
			j=0;
		}while(length > 0);
	}else{ /* Do the 16 bit version */
		register Uint16 *outData16Left = (Uint16 *)0x25a10000;
		register Uint16 *outData16Right = (Uint16 *)(0x25a10000 + (BUFSIZE*2));
	    register Uint16 temp;
	    
		outData16Left += j;
		outData16Right += j;	
		do{
			if (length + j > BUFSIZE)
				numToGet = BUFSIZE - j;
			else
				numToGet = length;			
			
			length -= numToGet;	
	        
			switch(format){                                 
			
				case 2: /* 16 bit mono to 16 bit stereo */
					if (data != NULL){ 
						while(numToGet--){               
							temp = (Uint16)*data++;
							temp |= (Uint16)*data++ << 8; 
							*outData16Left++ = temp;
						}
					}else
						while(numToGet--) *outData16Left++  = 0;						
					break;
				case 3: /* pure 16 bit stereo */
					if (data != NULL){ 
						while(numToGet--){                
							temp = (Uint16)*data++;
							temp |= (Uint16)*data++ << 8; 	
							*outData16Left++ = temp;
							temp = (Uint16)*data++;     
							temp |= (Uint16)*data++ << 8;
							*outData16Right++ = temp;
						}
					}else
						while(numToGet--) *outData16Right = *outData16Left++  = 0;						
					break;
			}                                               
			outData16Left = (Uint16 *)0x25a10000; outData16Right = (Uint16 *)(0x25a10000 + (BUFSIZE*2));
			j=0;
		}while(length > 0);
	}
}
loadDUKData(DUCK_HANDLE dckPtr, int length,int format)
{	
	int j,numToGet,numGotten = 0;
	Uint16 *outData16Left = (Uint16 *)0x25a10000,*outData16Right = (Uint16 *)(0x25a10000 + (BUFSIZE*2));

	j = lastpos % BUFSIZE; 
	
	outData16Left += j;	 
	outData16Right += j;	 
		
	do{
		if (length + j > BUFSIZE)
			numToGet = BUFSIZE - j;
		else
			numToGet = length;			
		
		length -= numToGet;	

       	if (NumChannels == 2){
			numGotten += getNextStereoSample(dckPtr,outData16Left,outData16Right,numToGet);
		}else
			numGotten += getNextMonoSample(dckPtr,outData16Left,numToGet);

		outData16Left = (Uint16 *)0x25a10000; outData16Right = (Uint16 *)(0x25a10000 + (BUFSIZE*2));
		j=0;
	}while(length > 0);
	
	lastpos += numGotten;	
}

void loadSamples(int SmplsPerFrame)
{
    static Uint8 *aData = NULL;
    static unsigned long chunkSamples;
	static int bps[] = {1,2,2,4};	
    int format;

#if AUDIO
	
	if (wFormat != 1){
		loadDUKData(duck_handle, SmplsPerFrame,wFormat);
		return;
	}
	
	if((BytesPerSec/NumChannels) == SamplesPerSec)
		format = 0;
	else
		format = 2;
	
	format += NumChannels-1;
	
	/* PCM audio needs no decompression, just load it to the buffer */
	do{
	    if (aChunkIndex < 0){          
	    	aChunkIndex = getNextAudio(duck_handle,(void **)&aData,&chunkSamples);
			if (aChunkIndex < 0){
				if (aChunkIndex == -1){
					chunkSamples = 2000; /* arbitrary number of empty samples */
					aData = NULL;
				}
				else return;
			}
			chunkSamples /= bps[format];
		}
		if (chunkSamples <= SmplsPerFrame){           
			loadPCMData(aData,chunkSamples,format);
			SmplsPerFrame -= chunkSamples;
			releaseChunk(duck_handle,aChunkIndex,0);
			aChunkIndex = -1;
		}else{                                          
			loadPCMData(aData,SmplsPerFrame,format);  
			chunkSamples -= SmplsPerFrame;
			if (aData != NULL) aData += SmplsPerFrame*bps[format];
			SmplsPerFrame = 0;
		}
	}while (SmplsPerFrame > 0);
	
#endif

}   

void resetFrameNum(void)
{
	targetFrameSample = GlobalBPF;
	
	if (SamplesPerSec == 44100)    
		targetFrameSample += GlobalBPF;
		
	targetSample = totalSamples = 0;
	
	*fPtr = 0;                                           
}

SndPcmPlayAdr my_pcm_adr;
static Uint8 last_radr;   

/* loadAudio keeps time and transfers audio to the sound processor's buffer*/

void loadAudio(int playing)
{
    register long SmplsPerFrame;
    static int tick = 0;       
    Uint8 llradr;
    
	if (!playing){
		resetFrameNum();
		return;
	}
    
    if (SamplesPerSec == 44100)                 
		totalSamples += (SmplsPerFrame = 735);		
	else
		totalSamples += (SmplsPerFrame = 367 + (tick ^= 1));                                        

#if AUDIO
    if (playAudio){
	    /* based on where it reports the play counter to be (4K sample granular),
	       correct for drift between the video and audio subsystems, this keeps the 
	       video in sync with the audio, Tom Miley at Sega mentioned how the audio 
	       rates may drift with loading */
	       
		SND_GetPcmPlayAdr(&my_pcm_adr, 2);
		
		if ((llradr = SND_PCM_RADR(my_pcm_adr)) != last_radr){
			last_radr = llradr;
			
			targetSample += 0x1000;
	
			if (targetSample > totalSamples){
				/*DIAGNOSTIC(DBG_Printf("behind by %d\n",targetSample - totalSamples);)*/
				SmplsPerFrame += targetSample - totalSamples;
			}else if (totalSamples > targetSample){
				/*DIAGNOSTIC(DBG_Printf("ahead by %d\n",totalSamples - targetSample);)*/
				SmplsPerFrame -= totalSamples - targetSample;
			}										
			totalSamples = targetSample;         
		}
		/*
	    if (SmplsPerFrame > 0)
			loadSamples(SmplsPerFrame);		
		*/
	}
#endif   

	/**** now see if we need to update the frame counter ****/
    while (totalSamples >= targetFrameSample){
    
		targetFrameSample += GlobalBPF;
	
	    if (SamplesPerSec == 44100)	/* do it again if running at 44K */    
			targetFrameSample += GlobalBPF;
		
		(*fPtr)++;
		if (modFPS[GlobalFrameRate] && (GlobalBPF == samplesPerFPS[GlobalFrameRate])){
		    targetFrameSample += !((*fPtr) % modFPS[GlobalFrameRate]);
		    
		    if (SamplesPerSec == 44100)  /* do it again if running at 44K */
		    	targetFrameSample += !((*fPtr) % modFPS[GlobalFrameRate]);	    	          
		}
	}
	if (!playing) resetFrameNum();
}

void duckAudioInitHW(void)
{
    int i;
	SndIniDt sys_ini;

    fPtr = (volatile long *)((Uint32)&frameNum | 0x20000000);           
					   
#if AUDIO
    /* load the sound processor's program and map */				    

	SND_INI_PRG_ADR(sys_ini) = (Uint16 *)&scsp_sound_driver;
	SND_INI_PRG_SZ(sys_ini) = (Uint16)((Uint32)&scsp_sound_driver_end - (Uint32)&scsp_sound_driver);
	SND_INI_ARA_ADR(sys_ini) = (Uint16 *)&scsp_map;
	SND_INI_ARA_SZ(sys_ini) = (Uint16)((Uint32)&scsp_map_end - (Uint32)&scsp_map);
    
   	DIAGNOSTIC(DBG_Printf("Calling SND_Init...\n");)
    SND_Init(&sys_ini);		/* Initialize the sound system */

   	DIAGNOSTIC(DBG_Printf("Calling SND_ChgMap...\n");)
    SND_ChgMap(0);			/* change to the (only) map */   

#endif                            
}
/* main sound kickoff routine  */
         
int prebuffer = 0;
         
void duckAudioInitSW(void)
{   
	int i;

    getFrameRates(duck_handle, &GlobalFrameRate, &GlobalBPF, &prebuffer);    

	aChunkIndex = -1;
	lastpos = 0;
	
    playAudio = (getAudioInfo(duck_handle, &NumChannels, &SamplesPerSec, &BytesPerSec, &wFormat) != NULL);
    
	if (playAudio){
		if (wFormat > 1){
			resetStereo4Bit(duck_handle);
			resetMono4Bit(duck_handle);
		}
		DIAGNOSTIC(DBG_Printf("loading first audio\n");)
	    loadSamples(prebuffer);	/* get prebuffered samples */
		DIAGNOSTIC(DBG_Printf("audio primed.\n");)
	}
}

SndPcmChgPrm pcm_chg;
SndPcmStartPrm pcm_start;

void pauseAudio(void)
{
	if (playAudio) SND_StopPcm(2);
}

void resumeAudio(void)
{
	if (playAudio) SND_StartPcm(&pcm_start, &pcm_chg);
}

void duckAudioStart(void)
{
    
#if AUDIO
	if (playAudio){
		/* fill in the PCM start and change structures */
	    
	    if((BytesPerSec/SamplesPerSec) == NumChannels) 
			SND_PRM_MODE(pcm_start) = SND_MD_8;
		else 
			SND_PRM_MODE(pcm_start) = SND_MD_16;
	    
	    if (NumChannels == 2)
			SND_PRM_MODE(pcm_start) |= SND_MD_STEREO;
	    else                           
			SND_PRM_MODE(pcm_start) |= SND_MD_MONO;
	    
	    if (wFormat > 1){
			if (NumChannels == 1)
				SND_PRM_MODE(pcm_start) = SND_MD_16 | SND_MD_MONO;
		    else
		    	SND_PRM_MODE(pcm_start) = SND_MD_16 | SND_MD_STEREO;	    
		}                                                                   

	    SND_PRM_SADR(pcm_start) = 0x1000;
	    SND_PRM_SIZE(pcm_start) = BUFSIZE;   /* ~36k buffer x2 */
		SND_PRM_NUM(pcm_chg) = 2;
		SND_PRM_LEV(pcm_chg) = 7;
		SND_PRM_PAN(pcm_chg) = 0;
		SND_PRM_PICH(pcm_chg) = (SamplesPerSec == 44100) ? 0 : 0x7800; /* set play speed 22 or 44 khz*/
		SND_L_EFCT_IN(pcm_chg) = 0;
		SND_L_EFCT_LEV(pcm_chg) = 0;
		SND_R_EFCT_IN(pcm_chg) = 0;
		SND_R_EFCT_LEV(pcm_chg) =  0;
	    
	    DIAGNOSTIC(DBG_Printf("calling StartPCM\n");)
	    /* get this show on the road with the SND_StartPcm command */
		SND_StartPcm(&pcm_start, &pcm_chg);
		SND_GetPcmPlayAdr(&my_pcm_adr, 2);
		last_radr = SND_PCM_RADR(my_pcm_adr);
	}
#endif
	
	resetFrameNum();
	playing = TRUE;
}
void duckAudioEnd(void)
{   
    SndPcmPlayAdr pcm_adr;

#if AUDIO    
	if (playAudio) SND_StopPcm(2);
#endif
	
	aChunkIndex = -1;
	lastpos = 0;
}
