/*******************************************************************************\
*                                                                              *
* dukfile.c -   file management code for SegaSaturn TrueMotion/S player        *
*               Copyright (c) 1994,95, The Duck Corp. All rights reserved.     * 
*                                                                              *
*                 written by:       Eric Ameres                                *
*                 Version:      .985                                           *
*                 Created:       03/07/95                                      *
*                                                                              *
********************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
                     
#include <sega_xpt.h>
#include <sega_dbg.h>
#include <sega_int.h> 
#include <sega_gfs.h>
#include "duksega.h"

#define SMCD_MAX_OPEN  1
#define SMCD_MAX_DIR   64
#define SMCD_SECT_SIZE 2048
#define SMCD_RD_SIZE   1

Sint32     SMCD_gfsret;
Sint32     SMCD_readflag;
Uint8      SMCD_gfslib_work[GFS_WORK_SIZE(SMCD_MAX_OPEN)];
GfsDirTbl  SMCD_dirtbl;

#undef DEBUG
#define DEBUG 0

#if DEBUG
#define DIAGNOSTIC(x) x
#else 
#define DIAGNOSTIC(x)
#endif              
                    
/*Uint32 dkSectSize = 2048;*/
#define dkSectSize 2048
#define BF_BYTES dkSectSize

Uint32 duckBuf[2048/4];
Uint8 *duckBPtr = (Uint8 *)duckBuf;

int numInBlock,nextToGet;
Sint32 tfid,fn,fsize,atr;

/******* initialize file system **********/

long duckInitFileSys(GfsDirId *dirID,GfsDirName *dName,int dirMax)
{     
	duckBPtr = (Uint8 *)((Uint32)duckBuf | 0x20000000); 

	if (dirMax != -1){
		GFS_DIRTBL_TYPE(&SMCD_dirtbl) = GFS_DIR_NAME;    
		GFS_DIRTBL_NDIR(&SMCD_dirtbl) = dirMax;
		GFS_DIRTBL_DIRNAME(&SMCD_dirtbl) = dName;
	}
	SMCD_gfsret = GFS_Init(SMCD_MAX_OPEN, &SMCD_gfslib_work, &SMCD_dirtbl);
	      
	return SMCD_gfsret;
}

/* used when the parser wants to read index before beginning streaming */

void gooseCD(GfsHn gfs,int size)
{
	GFS_CdMovePickup(gfs);
	GFS_NwCdRead(gfs,size >>11);
    GFS_NwExecOne(gfs); /* really kickoff the cd read */
}       

Sint32 oldMode,oldParam;

/* once playback is really underway, start the CD streaming */	
static void duckCdRead(GfsHn gfs)
{   
	GFS_SetTransPara(gfs,4);
	oldMode = GFS_SetTmode(gfs,GFS_TMODE_SCU);

	GFS_CdMovePickup(gfs);
		
	if (GFS_NwCdRead(gfs,440000)) return;
	
    GFS_NwExecOne(gfs); /* really kickoff the cd read */                        
}   

/* open the named file */
GfsHn duck_open(const char *fname)
{
	static Sint32 fid = 0;
	Sint32 dirid;
	GfsHn gfs;
	int tries = 0;
	const char *nextSect = fname;
	char dirname[16];
	
	nextToGet = numInBlock = 0;

/****** descend into the proper directory *****/
    
    if (strcspn(nextSect,"\\") == 0)
	nextSect++;
    
    while(strcspn(nextSect,"\\") != strlen(nextSect)){ 
		strncpy(dirname,nextSect,strcspn(nextSect,"\\"));
		dirname[strcspn(nextSect,"\\")] = 0;            
		nextSect += strcspn(nextSect,"\\") + 1;
		dirid = GFS_NameToId((Uint8 *)dirname);            
		GFS_LoadDir(dirid,&SMCD_dirtbl);
		GFS_SetDir(&SMCD_dirtbl);
    }

/******* allows playing of all DUK or AVI files in the current directory *******/    
    if (!strcmp(nextSect,"PLAYALL.DUK")){
	    while((strstr((const char *)GFS_IdToName(fid),".DUK") == NULL) && (tries++ < 32)){
			if (fid++ > 32) fid = 0;                    
		}
		if (tries >= 32) return (GfsHn)-1;      
	}else if (!strcmp(nextSect,"PLAYALL.AVI")){
	    while((strstr((const char *)GFS_IdToName(fid),".AVI") == NULL) && (tries++ < 32)){
			if (fid++ > 32) fid = 0;                    
		}
		if (tries >= 32) return (GfsHn)-1;      
	}else 
/****** if not the special case, then check for a normal file *****/	
	if ((fid = GFS_NameToId((Uint8 *)nextSect)) < 0){
		DIAGNOSTIC(DBG_Printf("error finding id %d\n",fid);)
		return (GfsHn)-1;                       
	}     
	
	if ((gfs = GFS_Open(fid)) == NULL){
		DIAGNOSTIC(DBG_Printf("error opening file id %d\n",fid);)
		fid = 0;                
		return (GfsHn)-1;
	}
    fid++;
    
    GFS_GetFileInfo(gfs,&tfid,&fn,&fsize,&atr);
    
    /*                                                 
    dkSectSize = (atr & GFS_ATR_FORM1) ? 2048 : 2324;
    dkSectSize = 2048;
    */
	
	GFS_CdMovePickup(gfs);
	GFS_SetTransPara(gfs,1);
    oldMode = GFS_SetTmode(gfs,GFS_TMODE_SCU);
	
    GFS_NwExecOne(gfs); /* really kickoff the cd read */

	return gfs;
}

/* close a file */
void duck_close(GfsHn gfs)
{   
	GFS_NwStop(gfs);        /* stop access to file */                             
    GFS_Close(gfs);
}

/* read a number of bytes from a file */
long duck_read(GfsHn ghndl,Uint8 *buf, Uint32 nbytes)
{   
	Sint32 backUp;
	int totalRead = 0;
	int numRead=0,desired=0;        
	    
    if (!nbytes) return 0;
    
    /****************************************************************************
	first, check to see if any bytes already waiting in short read buffer
    ****************************************************************************/
    
	if (nextToGet != numInBlock){
		int amountToGet = nbytes;
		
		if (amountToGet >= (numInBlock - nextToGet))
			amountToGet = (numInBlock - nextToGet);
	
		if (buf != NULL){       
			memcpy(buf,&duckBPtr[nextToGet],amountToGet);
			buf += amountToGet;
		}                                  
		totalRead = amountToGet;
		nextToGet += amountToGet;
		nbytes -= amountToGet;
	}               
    
    /*****************************************************************************
	next, read any full sectors requested directly into the specified buffer
    *****************************************************************************/
	
	if (nbytes >= dkSectSize){
		int bytesNeeded;
		
		desired = bytesNeeded = (nbytes>>11)<<11;
		
		DIAGNOSTIC(DBG_Printf("getting %d in full sectors\n",bytesNeeded);)
					     
		if (buf == NULL){
			GFS_Seek(ghndl, bytesNeeded >> 11, GFS_SEEK_CUR);
			numRead = bytesNeeded;
		}else{
										     
			backUp = GFS_Tell(ghndl);               
									
			numRead = GFS_Fread(ghndl, bytesNeeded >> 11, buf, bytesNeeded);
			
			if (numRead < 0) return numRead;
			while (numRead != bytesNeeded){
				if (GFS_IsEof(ghndl)) break;
				
				GFS_Seek(ghndl, backUp, GFS_SEEK_SET);                  
				numRead = GFS_Fread(ghndl, bytesNeeded >> 11, buf, bytesNeeded);
				if (numRead < 0) return numRead;
			} 
			buf += numRead;
		}
		totalRead += numRead;
		nbytes -= numRead;
	}
	
    /*************************************************************
	if we need any more bytes, we have to read a sector into 
	the short buffer and hand back the requested number
    *************************************************************/
	
	if (nbytes > 0){                                                                  
		backUp = GFS_Tell(ghndl);                               
		numInBlock = GFS_Fread(ghndl, 1, duckBPtr, dkSectSize);
	
		if (numInBlock < 0) return numInBlock;
		while (numInBlock != dkSectSize){
			if (GFS_IsEof(ghndl)) break;
			
			GFS_Seek(ghndl, backUp, GFS_SEEK_SET);                  
			numInBlock = GFS_Fread(ghndl, 1, duckBPtr, dkSectSize);
			if (numInBlock < 0) return numInBlock;
		}
		if (numInBlock < nbytes) nbytes = numInBlock;
		
		if (buf != NULL)
			memcpy(buf,duckBPtr,nbytes);
			
		nextToGet = nbytes;
		totalRead += nbytes;
	}
	return totalRead;
}

/* seek to a certain location within the file */
/* only supports seek from beginning of file or current location */

long duck_seek(GfsHn gHndl,long offs, int origin)
{       
	if (origin == SEEK_SET){
		GFS_NwStop(gHndl);        /* stop access to file */                             
    	while(!GFS_NwIsComplete(gHndl));
		
    	GFS_Seek(gHndl, offs>>11, GFS_SEEK_SET);
	    while(!GFS_NwIsComplete(gHndl));
		
		duckCdRead(gHndl);
					   
		nextToGet = numInBlock = 0;
		
		if (offs % dkSectSize)
			duck_read(gHndl,NULL, offs % dkSectSize);
			
		return offs;
	}else if (origin == SEEK_CUR){                               
	    duck_read(gHndl, NULL, offs);                
		return ((GFS_Seek(gHndl, 0, GFS_SEEK_CUR) - 1) << 11) + nextToGet;
	}
	return -1;
}
