/*******************************************************************************\
*                                                                               *
* dukplay.c -   TrueMotion/S player code for Sega Saturn                        *
*               Copyright (c) 1994,95, The Duck Corp. All rights reserved.      * 
*                                                                               *
*                 modified by:       Eric Ameres                                *
*                 Version:      .982                                             *
*                 Edited:       03/07/95                                         *
					- switched entirely to new SOA and SOJ libs
					- support complete for new GNU and Hitachi tools
					- tools used for this build:
						gcc version cygnus-2.6.0-941014-SOA-950215
						SH SERIES C Compiler Ver. 2.0(Release05B)
                    - dukaudio.c - corrected problems with 16 bit PCM audio
					- added HIGH_Y color mode for interframe decompression
*                 Version:      .98                                             *
*                 Edited:       02/15/95                                        *
					- compiled with the new Hitachi libs and includes
					- new gnu stuff to follow... as we're updated by SOA
					- dukaudio.c streamlined, support for 16Bit PCM added
					- new gnu stuff to follow... as we're updated by SOA
					  overhead to the slave CPU, once running, the master 
					  handles CD I/O (little running overhead) and interrupt 
					  stuff (audio, sprite flipping, etc...)
					
*                 Version:      .97                                             *
*                 Edited:       01/25/95                                        *
					- added support for Hitachi tool set
					- implemented normal color interframe
					- added 4 bit duk4Bit (4:1 compressed 16 bit mono & stereo) audio
					- re-engineered main play loop to reflect new "no-skip" policy
					- added adaptive datarate fill buffer library 
					- correctly re-implemented triple screen buffering
*                 Edited:       12/07/94                                        *
					- added support for high resolution color
					- added 8 bit stereo support to dukaudio.c
					- fixed end of audio glitch in dukaudio.c                   
					- added support for non-integer frame rates
					- added dynamic sprite allocation
					- improved file open result checking
					- fixed "curtains"
*                                                                               *
*                 Author:       Eric Ameres                                     *
*                 Version:      .95                                             *
*                 Created:      10/24/94                                        *
*                                                                               *
\*******************************************************************************/
#include        <stdio.h>
#include        <stdlib.h>
#include		<machine.h>

#define         _SPR2_       
#include        <sega_spr.h>
#include        <sega_dbg.h>
#include        <sega_xpt.h>
#include        <sega_scl.h>
#include        <sega_tim.h>
	                         
#include        <sega_int.h> 
                             
#include        <sega_csh.h>
#include        <sega_gfs.h>

#include        "duckfile.h"
#include        "duksega.h"

#if INCLUDE_PAD
#include        "..\pad\smp_pad.h"
#endif

#define sHeight 240
#define sWidth  320

#if DEBUG
#define DIAGNOSTIC(x) x
#else 
#define DIAGNOSTIC(x)
#endif

#define charSize(x, y) ((Uint16)( ((x >> 3) << 8) | (y)) )
#define charAddr(a) ((Uint16)((a-VDP1_VRAM_ADDR)>>3))
	
/* my command table for VDP1 */
SprSpCmd  boingo[] = {
	{            
	/* control  */ (JUMP_NEXT | FUNC_SCLIP),
	/* link     */ 0,
	/* drawMode */ 0,
	/* color    */ 0,
	/* charAddr */ 0,
	/* charSize */ 0,
	/* ax, ay   */ 0, 0,
	/* bx, by   */ 0, 0,
	/* cx, cy   */ sWidth-1, sHeight-1,
	/* dx, dy   */ 0, 0,
	/* grshAddr */ 0,
	/* dummy    */ 0
	},
	{
	/* control  */ (JUMP_NEXT | FUNC_LCOORD),
	/* link     */ 0,
	/* drawMode */ 0,
	/* color    */ 0,
	/* charAddr */ 0,
	/* charSize */ 0,
	/* ax, ay   */ 0, 0,
	/* bx, by   */ 0, 0,
	/* cx, cy   */ 0, 0,
	/* dx, dy   */ 0, 0,
	/* grshAddr */ 0,
	/* dummy    */ 0
	},
	/* 
	 * these are the default values for the video sprite
	 * they are now actually computed based on the frame size
	 */
	{
	/* control  */ (JUMP_NEXT | ZOOM_NOPOINT | DIR_NOREV | FUNC_DISTORSP),
	/* link     */ 0,
	/* drawMode */ COLOR_5 /*| SPD_DISABLE */,
	/* color    */ 0,
	/* charAddr */ 0x8000,
	/* charSize */ charSize(sWidth,sHeight),
	/* ax, ay   */ 0, 0,
	/* bx, by   */ sWidth-1, 0,
	/* cx, cy   */ sWidth-1, sHeight-1,
	/* dx, dy   */ 0, sHeight-1,
	/* grshAddr */ NO_GOUR,
	/* dummy    */ 0
	},
	{
	/* control  */ (JUMP_NEXT | ZOOM_NOPOINT | DIR_NOREV | FUNC_DISTORSP),
	/* link     */ 0,
	/* drawMode */ COLOR_5 /*| SPD_DISABLE*/,
	/* color    */ 0,
	/* charAddr */ 0x8000,
	/* charSize */ charSize(sWidth/2,sHeight/2),
	/* ax, ay   */ 0, 0,
	/* bx, by   */ sWidth/2-1, 0,
	/* cx, cy   */ sWidth/2-1, sHeight/2-1,
	/* dx, dy   */ 0, sHeight/2-1,
	/* grshAddr */ NO_GOUR,
	/* dummy    */ 0
	},
	{/* END      */
	/* control  */ CTRL_END,
	/* link     */ 0,
	/* drawMode */ 0,
	/* color    */ 0,
	/* charAddr */ 0,
	/* charSize */ 0,
	/* ax, ay   */ 0,
	/* bx, by   */ 0,
	/* cx, cy   */ 0,
	/* dx, dy   */ 0,
	/* grshAddr */ 0,
	/* dummy    */ 0
	}
};
long duckResult;
DUCK_HANDLE duck_handle,ducktemp;

Uint8  *VRAM;      
extern int playing, playAudio;
extern volatile int paused;

#if INCLUDE_PAD
extern volatile Sint16 SMPA_pad_data1;          /* 1P PAD DATA          */
extern volatile Sint16 SMPA_pad_data2;          /* 2P PAD DATA          */
#endif

typedef struct SPFRAME {
	int index;
	Uint32 *data;
	unsigned long width;
	unsigned long height;
	unsigned long length;
	volatile unsigned int spriteToDraw;
	unsigned long spwidth;
	unsigned long spheight;
	unsigned long xoff,yoff;
} spFrame;

spFrame bg,sp,sp2;

extern unsigned int GlobalFrameRate,SamplesPerSec;

static long currentFrame = 1;        

/*  
	we use 3 "sprites" so that 
	we never have to explicitly
	wait for the vertical blank
	sizes and addresses are now computed based on frame size.       
*/

#define SCREENBUFFERS 2
                             
Uint32 sprFunkyAddrTable[SCREENBUFFERS+1];
volatile Uint32 *staticAddrTable[SCREENBUFFERS+1];

/*******************************************************/
/*     write through addresses for shared vaiables     */
/*******************************************************/

volatile Uint32 **sprNormalAddrTable = staticAddrTable;

volatile unsigned int spriteDX;
volatile unsigned int *spritePtr = &spriteDX;
#define spriteToDX (*spritePtr)

volatile Uint32 swapper = 0;                        
volatile Uint32 *swapPtr = &swapper;
#define swapScrn (*swapPtr)                                         
extern volatile long *fPtr;    

/*******************************************************/

int errCount,drawErr;

void duckVideoInit1(void)
{
	Uint16 PAD_info_play1;
                                                                                       
    swapPtr = (Uint32 *)((Uint32)swapPtr | 0x20000000);           
    spritePtr = (unsigned int *)((Uint32)spritePtr | 0x20000000);                      
    sprNormalAddrTable = (volatile Uint32 **)((Uint32)sprNormalAddrTable | 0x20000000);
    
    /* enable interrupts */
	set_imask(0);
	       
	/* normal VDP2 init stuff lifted from SOJ demo code */
	SCL_Vdp2Init();
	SCL_SetPriority(SCL_SP0|SCL_SP1|SCL_SP2|SCL_SP3|
			SCL_SP4|SCL_SP5|SCL_SP6|SCL_SP7,7);
	SCL_SetSpriteMode(SCL_TYPE1,SCL_MIX,SCL_SP_WINDOW);
	SCL_SetColRamMode(SCL_CRM15_2048);

	/* our vertical blank init (see dukvblnk.c) */
	duckSetVBlank();
	
#if DEBUG
	/* if we are debuggin, initialize and tune in */
	DBG_Initial(&PAD_info_play1,RGB16_COLOR(31,31,31),0);
	DBG_DisplayOn();
	DBG_Printf("Hello Saturn!\n");
#endif

#if VIDEO
	SPR_Initial(&VRAM);
	SCL_VblInit();
	SCL_SetFrameInterval(-1);
	
	SCL_DisplayFrame();
	SCL_DisplayFrame();
	
	SCL_SetDisplayMode(SCL_NON_INTER,SCL_240LINE,SCL_NORMAL_A);
	SPR_SetTvMode(SPR_TV_NORMAL,SPR_TV_320X240,OFF);
	SPR_SetEraseData(0x8000,0,0,sWidth-1,sHeight-1);

	SCL_SetFrameInterval(0);

	SCL_Vdp2Init();
	SCL_SetPriority(SCL_SP0|SCL_SP1|SCL_SP2|SCL_SP3|
			 SCL_SP4|SCL_SP5|SCL_SP6|SCL_SP7,7);
	SCL_SetSpriteMode(SCL_TYPE1,SCL_MIX,SCL_SP_WINDOW);
	SCL_SetColRamMode(SCL_CRM15_2048);
	SCL_SetFrameInterval(0);
	
	SCL_SetDisplayMode(SCL_NON_INTER,SCL_240LINE,SCL_NORMAL_A);
	SPR_SetTvMode(SPR_TV_NORMAL,SPR_TV_320X240,OFF);
	SPR_SetEraseData(0x8000,0,0,sWidth-1,sHeight-1);
#endif
}

/* initialize a sprite to blackness */
void clearSprite(volatile register Uint32 *screen)
{   
	register Uint32 x;      
	
	x = (bg.spwidth*bg.spheight)/2;
	
	while(x--) *(screen++) = 0x80008000;
} 

/* 
 * play frames is currently the main play loop 
 * it gets called by DKAVIFilePlay
 */

/**********************************************************
  swapSprites is called by the sprite draw end interrupt,
  after drawing a sprite, check to see if the time has advanced 
  and that a new decompressed frame is available,
  if so, point the sprite address to the next available sprite
  
  by living in the sprite draw end rountine we shouldn't have to 
  worry about tearing screens in the middle of decompressing
**********************************************************/
extern int Erasing;

void swapSprites(void)
{
#ifdef __GNUC__
asm("
	sts.l	macl,@-r15
	sts.l	mach,@-r15
");            
#endif

#if SYNC
	if(currentFrame < *fPtr){
#endif
		if (swapScrn > 1){
			                                                                   
			swapScrn--;		/* release last frame used */
			currentFrame++;                    
#if VIDEO
			if (++bg.spriteToDraw > (SCREENBUFFERS - 1)) bg.spriteToDraw = 0;            
			boingo[3].charAddr = boingo[2].charAddr = sprFunkyAddrTable[bg.spriteToDraw];
			
			memcpy(VRAM,boingo,sizeof(boingo));
			if (Erasing)
				if (!--Erasing)	SPR_SetEraseData(0x8000,0,0,0,0);
#endif
		}
#if SYNC
    }   
#endif
    else if (paused){
			memcpy(VRAM,boingo,sizeof(boingo));
		if (Erasing)
			if (!--Erasing)	SPR_SetEraseData(0x8000,0,0,0,0);
	}
    	
#ifdef __GNUC__
asm("                 
	lds.l	@r15+,mach
	lds.l	@r15+,macl
");            
#endif
}


extern Sint32  VBInterval;   /* V-Blank Interval for Frame Change */
extern volatile Sint32  ReqDisplayFlag;   /* Display Request flag              */
extern volatile Sint32  ReqDisplayFlag2;   /* Display Request flag              */

int errFrame = -1,errData;

/* main play routine/loop */
void playFrames()
{                                    
	unsigned long length,audioFrame,videoFrame;
	int i,spf;   
#if TWOPROC
	extern volatile int *slaveStatus;
	extern void InitSlaveSH(void);
    /* initalize the slave SH2 */
    
	InitSlaveSH();
#endif
    
    while(paused); /* if somebody hit the pause key */

	/* initialize counters */
	             
	videoFrame = audioFrame = currentFrame = 0;
	swapScrn = 0;
    drawErr = errCount = 0;
    spf = getSamplesPerFrame(duck_handle);

	/*********************************
	 	pre-decompress 3 frames 
	 		into VDP1 VRAM
	 	  (treble buffering)
	**********************************/
    DIAGNOSTIC(DBG_Printf("Starting to decompress\n");)

#if DECOMPRESSION	
	dxFrame((Uint8 *)bg.data,(Uint8 *)(sprNormalAddrTable[0])); 
#endif
	releaseChunk(duck_handle,bg.index,0);
	fillBuffer(duck_handle,DCK_DEFAULT_READSIZE,0);	
	
	for (i = 1; i < SCREENBUFFERS; i++){	                              
		bg.index = getNextFrame(duck_handle,(void **)&bg.data,&bg.length);
#if DECOMPRESSION	
#if VDP1SPRITE
		dxFrame((Uint8 *)bg.data,(Uint8 *)(sprNormalAddrTable[i]));		 
#else
		setDestInfo(512,224,0,0);
		dxFrame((Uint8 *)bg.data,(Uint8 *)0x25c80000);
#endif		
#endif
		releaseChunk(duck_handle,bg.index,0);
		fillBuffer(duck_handle,DCK_DEFAULT_READSIZE,0);	
	}
    spriteToDX = bg.spriteToDraw = 0;
	
	/*********************************
	 fill in any gaps in the buffer   
	 (i.e. from the audio pre-buffering)
	**********************************/
	fillBuffer(duck_handle,64*1024,0);

	/*************************
     draw the first frame
    **************************/	              
	boingo[3].charAddr = boingo[2].charAddr = sprFunkyAddrTable[0];
	    
#if VIDEO && VDP1SPRITE
	DIAGNOSTIC(DBG_Printf("fire up first sprite\n");)
	memcpy(VRAM,boingo,sizeof(boingo));
#endif
	
	/*************************
    start the audio playing 
    **************************/
    DIAGNOSTIC(DBG_Printf("starting audio\n");)
	duckAudioStart();
	swapScrn = SCREENBUFFERS;
    
    DIAGNOSTIC(DBG_Printf("Starting play loop\n");)
    
#if TWOPROC
	go(duck_handle);
#endif	                                	

#if !VDP1SPRITE	
	SCL_SetFrameInterval(-1);
#endif
		
    do{ 
#if INCLUDE_PAD			
		if ((SMPA_pad_data1|SMPA_pad_data2) & PAD_X) break;	/* stop play with pad_x */
#endif	

#if VDP1SPRITE
#if !TWOPROC			
    	/*********************************************************
    		decompress into next buffer as soon as it's available
    	*********************************************************/

#if SYNC		              		
		if (swapScrn < SCREENBUFFERS){
#else
		{
#endif
			bg.index = getNextFrame(duck_handle,(void **)&bg.data,&bg.length);
			if (bg.index < 0){
				if ((bg.index == -1) && (swapScrn == 1)) break;
			}else{
#if DECOMPRESSION
				decodeFrame(bg.data, &bg.width, &bg.height);
				
				DIAGNOSTIC(DBG_Printf("Decompressing %d\n",bg.index);)
				if ((drawErr = dxFrame((Uint8 *)bg.data,(Uint8 *)(sprNormalAddrTable[spriteToDX]))) != 0) errCount++;
#endif				                                                                                                 
		        releaseChunk(duck_handle,bg.index,0);
				if (++spriteToDX > (SCREENBUFFERS-1)) spriteToDX = 0;
				swapScrn++;
			}		
		}
#else
        if (!*slaveStatus) break;
#endif           
#else
		if (videoFrame < *fPtr){
			videoFrame++;
			setDestInfo(512,224,0,0);
	
			if (!(ReqDisplayFlag || ReqDisplayFlag2)){
				bg.index = getNextFrame(duck_handle,(void **)&bg.data,&bg.length); 
				if (bg.index == -1) break;
				
				if (bg.index > -1) dxFrame((Uint8 *)bg.data,(Uint8 *)0x25c80000);		        
				
				ReqDisplayFlag = 1;        
				
				if (bg.index > -1) releaseChunk(duck_handle,bg.index,0);
			}
		}
#endif

#if SYNC
    	if (audioFrame < *fPtr){
	    	/* code to be called once per frame tick */
#else
		{
#endif
    		audioFrame++;
            
            /* call fillBuffer using default values */
			if ((duckResult = fillBuffer(duck_handle,DCK_DEFAULT_READSIZE,0)) < 0)	break;
			
			/*****************************************************
	    		Load audio buffer with 1 frame worth of samples
	    		(if not being done by vertical blank interrupt)
	    	*****************************************************/ 
    	    if (playAudio) loadSamples(spf);
		}		

    }while(1);
       
#if TWOPROC
    ShutDownSH();	
#endif

}

int DKAVIFilePlay(char *name,int bufferSize)	/* play initialization and call to playFrames */
{   
	int i=0;
	unsigned long length;
                       
	/* open and buffer the avi file */                                                     
	if (!(duckResult = DKAVIFileOpen(name,&duck_handle, bufferSize,128*1024,0))){  
#if VIDEO
		SPR_Initial(&VRAM);
		/* fire up VDP1 and VDP2 */
		/* during playback we use autoframe change mode */
		
		SCL_VblInit();
		SCL_SetFrameInterval(-1);
			
		SCL_DisplayFrame();
		SCL_DisplayFrame();
		
		SCL_SetDisplayMode(SCL_NON_INTER,SCL_240LINE,SCL_NORMAL_A);
		SPR_SetTvMode(SPR_TV_NORMAL,SPR_TV_320X240,OFF);
		SPR_SetEraseData(0x8000,0,0,sWidth-1,sHeight-1);
		SCL_SetFrameInterval(0);
	    
		SCL_Vdp2Init();
		SCL_SetPriority(SCL_SP0|SCL_SP1|SCL_SP2|SCL_SP3|
				 SCL_SP4|SCL_SP5|SCL_SP6|SCL_SP7,7);
		SCL_SetSpriteMode(SCL_TYPE1,SCL_MIX,SCL_SP_WINDOW);
		SCL_SetColRamMode(SCL_CRM15_2048);
		SCL_SetFrameInterval(0);
		
		SCL_SetDisplayMode(SCL_NON_INTER,SCL_240LINE,SCL_NORMAL_A);
		SPR_SetTvMode(SPR_TV_NORMAL,SPR_TV_320X240,OFF);
		SPR_SetEraseData(0x8000,0,0,0,0);
	    /*SPR_SetEraseData(0x8000,0,0,sWidth-1,sHeight-1);*/	
		Erasing = TRUE;
		
		SCL_VblInit();	/* initialize system vblank stuff */	
#endif

		DIAGNOSTIC(DBG_Printf("file opened & buffered\n");)

		duckAudioInitSW();	/* fire up the audio */		
		*fPtr = 0;			/* init the framecounter */
					       
		/* get the first frame */
		bg.index = getNextFrame(duck_handle,(void **)&bg.data,&bg.length);
		/* decode the frame to get the necessary sprite size and location */
		decodeFrame(bg.data, &bg.width, &bg.height);
		DIAGNOSTIC(DBG_Printf("frame info %dx%d\n",bg.width,bg.height);)
		
		setFrameBuffer(bg.data);
		
		bg.spwidth = bg.width;
		bg.spheight = bg.height;
		
		bg.xoff = (bg.spwidth - bg.width)/2;
		bg.yoff = (bg.spheight - bg.height)/2;
		
		setDestInfo(bg.spwidth,bg.spheight,bg.xoff,bg.yoff);
							  
		/* center the sprite on the screen and calculate sprite VRAM addresses*/
		{
			unsigned long firstadd;
			int m;                          
					    
			boingo[2].ax = boingo[2].dx = ((sWidth-1) - bg.spwidth)/2;
			boingo[2].bx = boingo[2].cx = bg.spwidth + boingo[2].ax -1;
			boingo[2].charSize = charSize(bg.spwidth,bg.spheight);
			firstadd = 0x80000;
			sprNormalAddrTable[SCREENBUFFERS]  = (Uint32)0L;
			sprFunkyAddrTable[SCREENBUFFERS] = (Uint32)0L;
			for (m = (SCREENBUFFERS-1); m > -1; m--){
				firstadd -= (bg.spwidth*bg.spheight)*2;
				sprNormalAddrTable[m] = (Uint32 *)(0x25c00000 + firstadd);
				sprFunkyAddrTable[m] = (Uint32)(firstadd/8);
			}                                                  
			boingo[2].ay = boingo[2].by = ((sHeight-1) - bg.spheight)/2;
			boingo[2].cy = boingo[2].dy = bg.spheight + boingo[2].ay -1;
			
			boingo[3].ax = boingo[3].dx = 20;
			boingo[3].bx = boingo[3].cx = bg.spwidth/4 + boingo[3].ax -1;
			boingo[3].charSize = charSize(bg.spwidth,bg.spheight);
			boingo[3].ay = boingo[3].by = 20;
			boingo[3].cy = boingo[3].dy = bg.spheight/4 + boingo[3].ay -1;
			
			i = 0;
		    
			while(sprNormalAddrTable[i] != NULL)
				clearSprite(sprNormalAddrTable[i++]);    
            
			boingo[3].control = CTRL_END;	
            boingo[4].control = CTRL_END;
		}                                                      

	    
	    playFrames();		/* let the somebody draw the frames */
		
		freeFrameBuffer();
		playing = FALSE;	/* stop the audio buffering */
		duckAudioEnd();		/* stop the audio driver */

#if VIDEO        
        SPR_SetEraseData(0x8000,0,0,sWidth-1,sHeight-1);
		
        /* erase the screen */
	    i = 0;
		    
		while(sprNormalAddrTable[i] != NULL)
			clearSprite(sprNormalAddrTable[i++]);    
		
		boingo[3].charAddr = boingo[2].charAddr = sprFunkyAddrTable[0];
		memcpy(VRAM,boingo,sizeof(boingo));
#endif
		
		DKAVIFileClose(duck_handle);	/* close avi file and release (free) allocated buffers */
		
	}else
		return -1;            
	
	return 0;
}    
/*  end of file dukplay.c */
