/*******************************************************************************\
*                                                                               *
* duksega.c -   TrueMotion/S player code for Sega Saturn                        *
*               Copyright (c) 1994, The Duck Corp. All rights reserved.         * 
*                                                                               *
*                 Editor:       Eric Ameres                                     *
*                 Version:      .97                                             *
*                 Edited:       01/25/95                                        *
					- added support for Hitachi tool set
					- implemented normal color interframe
					- added 4 bit duk4Bit (4:1 compressed 16 bit mono & stereo) audio
					- re-engineered main play loop to reflect new "no-skip" policy
					- added adaptive datarate fill buffer library 
					- correctly re-implemented triple screen buffering
*                 Edited:       12/07/94                                        *
					- added support for high resolution color
					- added 8 bit stereo support to dukaudio.c
					- fixed end of audio glitch in dukaudio.c                   
					- added support for non-integer frame rates
					- added dynamic sprite allocation
					- improved file open result checking
					- fixed "curtains"
*                                                                               *
*                 Author:       Eric Ameres                                     *
*                 Version:      .95                                             *
*                 Created:      10/24/94                                        *
*                                                                               *
\*******************************************************************************/
#include        <stdio.h>
#include        <stdlib.h> 
#include        <sega_xpt.h>
#include        <sega_gfs.h>
#include        <sega_dbg.h>

#include        "duckfile.h"
#include        "duksega.h"

#if DEBUG
#define DIAGNOSTIC(x) x
#else 
#define DIAGNOSTIC(x)
#endif

#define MEMLOW	0x26048000      
#define MEMLEFT(base) (0x26100000-base)

/* 
 *  main application "stub" 
 *  YOUR APPLICATION GOES HERE! 
 */

void main()
{
	int fError;        
	
	GfsDirId *dirIDs;
    GfsDirName *dirNames;
    
    /* set up memory pool starting at MEMLOW */
	/* notice the use of write through addressing since 
	 * this may be shared between master and slave in a dualproc player
	 *
	 * feel free to use your own memory manager (see dukmem.c) */
	MEM_Init(MEMLOW, MEMLEFT(MEMLOW));	/* grab all unused memory */
		
	duckVideoInit1();	/* video init stuff which only needs to be called once per app */	
	
	DIAGNOSTIC(DBG_Printf("Initializing Audio Hardware...");)

	duckAudioInitHW();	/* hardware audio init stuff needs to be called once per app */
    
    /* if DEBUG is defined as 1 in duksega.h, 
     * we can use DBG_Printf after calling duckVideoInit1 */
    
	DIAGNOSTIC(DBG_Printf("Initializing file system...");)
    
	/*	fire up the CD file system with room for 
	 *	64 files and/or directories per directory */
	
	dirIDs = (GfsDirId *)duck_calloc(64,sizeof(GfsDirId));
	dirNames = (GfsDirName *)duck_calloc(64,sizeof(GfsDirName));
	
	if ((fError = duckInitFileSys(dirIDs,dirNames,64)) > -1){
		DKAVIFilePlay("\\DUCK.LOG",476*1024);			
	    do{ 
#if 0	    
			/*
			 * the sample player lets the CD buffering use the 412K of memory left
			 * after reserving 280K for frame, indexing and other internal buffers 
			 * and data structures) */

			/* play files name duck[123].avi in the directory dir1 sequentially */
			/* if no directory is specified, play out of the current (last set) directory */
			                                           
			DKAVIFilePlay("\\DIR1\\DUCK1.AVI",412*1024);
			DKAVIFilePlay("\\DIR1\\DUCK2.AVI",412*1024);
			DKAVIFilePlay("\\DIR1\\DUCK3.AVI",412*1024);
#endif
			/* DEMONSTRATION OF A SPECIAL CASE!!!
			   play all files in the root directory
			   with the extension .avi */
			/*368*/   			   
	    	if (DKAVIFilePlay("\\PLAYALL.AVI",512*1024) < 0) break;
	    	else while(1) DKAVIFilePlay("\\PLAYALL.AVI",512*1024);
	    	
        }while(1); 
    }                   
	/* not actually called in the looping player but allocating 
	 * memory without a provision for freeing it would be bad form */
	duck_free(dirIDs);
	duck_free(dirNames);
}

