/*******************************************************************************\
*                                                                               *
* dukaudio.c -   TrueMotion/S driver code for slave SH2 CPU on Sega Saturn      *
*                                                                               *
*                 Author:	Eric Ameres                                         *
*                 Version:	.95                                                 *
*                 Created:	10/94                                               *
*                 Copyright (c) 1994, The Duck Corp. All rights reserved.       * 
*                                                                               *
\*******************************************************************************/

#include <sega_xpt.h>
#include <stdlib.h> 
#include "duckfile.h"
#include "duksega.h"

#define VBR   0x06000000
#define SETSINT   ((Sint32 (*)()) *((Uint32*)(VBR+0x310)))

Uint32 *SlaveCommand  = (Uint32 *)0;          /* MASTER to SLAVE command AREA */
Uint32 SlaveState    = (Uint32)0;             /* SLAVE to MASTER state  AREA */
Uint32 SlaveParam    = (Uint32)0;       /* MASTER to SLAVE parameter  AREA */
Uint32 *SlaveHandle = &SlaveParam;
void **SlaveSHEntry = (void **)0x6000250;       /* BOOT ROMs dispatch address */
Uint32 SlaveHeartBeat = 0;

volatile Uint8 *SMPC_COM = (Uint8 *)0x2010001F;   /* SMPC command register */
volatile Uint8 *SMPC_RET = (Uint8 *)0x2010005f;   /* SMPC result register */ 
volatile Uint8 *SMPC_SF  = (Uint8 *)0x20100063;   /* SMPC status flag */     
const Uint8 SMPC_SSHON  = 0x02;          /* SMPC slave SH on command */      
const Uint8 SMPC_SSHOFF = 0x03;          /* SMPC slave SH off command */     

int slaveStart = 0;

/*******************************************************/
/*     write through addresses for shared vaiables     */
/*******************************************************/
                                               
extern volatile Uint32 *sprNormalAddrTable;

extern volatile unsigned int *spritePtr;
#define spriteToDX (*spritePtr)

extern volatile Uint32 *swapPtr;
#define swapScrn (*swapPtr)
                    
extern volatile long *fPtr;    

/* ============ DECLARE FUNCTIONS =========== */
void InitSlaveSH(void);
void SlaveSHmain(void);

/* ============ DEFINE FUNCTIONS ============ */
/* -Change BOOT ROMs "slave SH dummy loop entry" To "SlaveSHmain entry" */
/* -Enable Slave SH with SMPC command SSHON                             */
/* -Then Slave SH start and Initialized by BOOT ROM code                */
/* -Finally Slave SH round a flag checking loop in SlaveSHmain          */

#if TWOPROC
void /* slave SH Initialize (RUNS on main SH) */
ShutDownSH(void)
{
    while((*SMPC_SF & 0x01) == 0x01);

    *SMPC_SF = 1;                 /* --- SMPC StatusFlag SET */

    *SMPC_COM = SMPC_SSHOFF;      /* --- Slave SH OFF SET */

    while((*SMPC_SF & 0x01) == 0x01);           
}
void /* slave SH Initialize (RUNS on main SH) */

InitSlaveSH(void)
{

    volatile Uint16 i;

    SlaveState = 0;                /* set RUNNING state */

    /* SlaveSH ǐZbgȈڡӀ */

	ShutDownSH();
	
	for(i = 0 ; i < 1100; i++);   /* slave reset assert length */

#if 0

    *(void **)SlaveSHEntry = (void *)&SlaveSHmain; /* dispatch address set */

#endif

	SETSINT(0x94,(void *)&SlaveSHmain);

    /* SlaveSH ǐZbgȃȊ */

    *SMPC_SF = 1;                 /* --- SMPC StatusFlag SET */

    *SMPC_COM = SMPC_SSHON;       /* --- Slave SH ON SET */

    while((*SMPC_SF & 0x01) == 0x01);    
}

/*******************************************************/
volatile int *slaveStatus,slaveStat;

void go(Uint32 duck_handle)
{
	SlaveParam = duck_handle;                                    
    slaveStatus = (volatile int *)(((Uint32)&slaveStat) | 0x20000000);
    SlaveHandle = (Uint32 *)(((Uint32)&SlaveParam) | 0x20000000);
	*(Uint16 *)0x21000000 = 0xffff; /* slave FRT inp invoke */
	*slaveStatus = 1;
}

void /* Slave SH2 main loop (RUNS on slave SH) */

  SlaveSHmain(void)

{
    long thissynch = -1;
	Uint16 count;
    int ret;
    int fChunkIndex;
    unsigned long length,fFrame;
    void *fData;
    
    const Uint32 RUNNING = 1;
    const Uint32 WAITING = 0;
 
     /* Wait until SlaveSHReqCode is set */

    /* then call function for SlaveSHReqCode */ 
    slaveStart = 1;
    
    set_imask(0xf);
    
    slaveStart = 2;
    
    *(Uint16 *)0xfffffee2 = 0x0000;  /* IPRA int disable */
    *(Uint16 *)0xfffffe60 = 0x0000;  /* IPRB int disable */ 
    *(Uint8 *)0xfffffe10  = 0x01;    /* TIER FRT INT disable */
    slaveStart = 3;
    
    while(1){
		/* Use "FRT InputCaptureFlag" Polling for wait command from Master */
		slaveStart = 4;
    	
		if((*(Uint8 *)0xfffffe11 & 0x80) == 0x80){
			    *(Uint8 *)0xfffffe11 = 0x00; /* FTCSR clear */         
		    do{ 
		    	SlaveHeartBeat++;

				if (swapScrn < 3){		
					fChunkIndex = getNextFrame(*SlaveHandle,(void **)&fData,&length);
					if (fChunkIndex < 0){
						if ((fChunkIndex == -1) && (swapScrn == 1)) break;
					}else{
						dxFrame((Uint8 *)fData,(Uint8 *)(sprNormalAddrTable[spriteToDX]));
		
						releaseChunk(*SlaveHandle,fChunkIndex,0);                  
						
						if (++spriteToDX > 2){ spriteToDX = 0; }
						swapScrn++;				
					}			
				} 
		    }while(1);
			*slaveStatus = 0;
	    }    
    }
}
#endif
