/*******************************************************************************\
*                                                                               *
* dukvblnk.c -   TrueMotion/S vertical blank driver code for Sega Saturn        *
*                                                                               *
*                 Author:	Eric Ameres                                         *
*                 Version:	.95                                                 *
*                 Created:	10/94                                               *
*                 Copyright (c) 1994, The Duck Corp. All rights reserved.       * 
*                                                                               *
\*******************************************************************************/
                     
#include "duckfile.h"
#include "duksega.h"
#define         _SPR2_
#include "sega_spr.h"
#include "sega_xpt.h" 
#include "sega_scl.h"
#include "sega_int.h"

#if INCLUDE_PAD
#include "..\pad\smp_pad.h"

typedef struct SPFRAME {
	int index;
	Uint32 *data;
	unsigned long width;
	unsigned long height;
	unsigned long length;
	volatile unsigned int spriteToDraw;
	unsigned long spwidth;
	unsigned long spheight;
	unsigned long xoff,yoff;
} spFrame;

extern spFrame bg,sp,sp2;

Uint16  PadData1  = 0x0000;
Uint16  PadData1E = 0x0000;
Uint16  PadData2  = 0x0000;
Uint16  PadData2E = 0x0000;

Uint32  PadWorkArea[7];
                   
#endif /* INCLUDE_PAD */
                   
int Erasing;

extern int playing,GlobalBPF;
extern DUCK_HANDLE duck_handle;

extern volatile Uint32 swapScrn;
extern SprSpCmd  boingo[];
extern Uint8  *VRAM;     
static int direction = 0,speed=1;

int vbs,vbe,spre;
volatile int paused = FALSE;

#if INCLUDE_PAD
void doPad(volatile Sint16 edge,volatile Sint16 data, spFrame *sprite)
{
	if (edge){
		switch(edge){
			case PAD_START:
				paused ^= TRUE;             
				if (playing){
					if(paused) pauseAudio();
					else resumeAudio();
				}
				break;
			case PAD_MODE:
				{
					int dummy;
                    
					boingo[3].control = CTRL_END;
			
					Erasing =2;
					SPR_SetEraseData(0x8000,0,0,320,240);            
					
					boingo[2].ax = boingo[2].dx = ((320-1) - bg.spwidth)/2;
					boingo[2].bx = boingo[2].cx = bg.spwidth + boingo[2].ax -1;
					boingo[2].ay = boingo[2].by = ((240-1) - bg.spheight)/2;
					boingo[2].cy = boingo[2].dy = bg.spheight + boingo[2].ay -1;
					speed = 1;
					getFrameRates(duck_handle, &dummy, &GlobalBPF, &dummy);    
				}
				break;
			case PAD_C:
				if (speed > -2){
					GlobalBPF <<= 1;
					speed--;
				}
				break; 
			case PAD_X:
				speed = 1;
				break;
			case PAD_Y: 
				if (speed < 3){
					GlobalBPF >>= 1;
					speed++;  
				}
				break;
			default:
				break;
		}
	}
	
	if (data){
		/* this stuff does some simple sprite manipulation */
		/* l/r up/down moves the sprite */
		/* Z simulates a roll */
		
		if (data & PAD_LEFT){
			/*if ((sprite->index > -1) && !paused){
				if (sprite->xoff > 2) sprite->xoff -= 2; 
			}else */
			if ((boingo[2].ax < 0) || (boingo[2].bx < 320)){
				Erasing  = 2;
				SPR_SetEraseData(0x8000,0,0,320,240);
				boingo[2].ax += 2;
				boingo[2].bx += 2;
						
				boingo[2].cx += 2;
				boingo[2].dx += 2;
			}
		}
		
		if (data & PAD_RIGHT){
			/*if ((sprite->index > -1) && !paused) sprite->xoff += 2;
			else */
			if ((boingo[2].ax > 0) || (boingo[2].bx > 320)){
				Erasing  = 2;			
				SPR_SetEraseData(0x8000,0,0,320,240);
				boingo[2].ax -= 2;
				boingo[2].bx -= 2;
					
				boingo[2].cx -= 2;
				boingo[2].dx -= 2;
			}
		}                                   
		
		if (data & PAD_UP){
			if ((boingo[2].ay > 0) || (boingo[2].dy > 224)){
				Erasing  = 2;			
				SPR_SetEraseData(0x8000,0,0,320,240);
				boingo[2].ay -= 2;
				boingo[2].dy -= 2;
					
				boingo[2].by -= 2;
				boingo[2].cy -= 2;
			}
		}

		if (data & PAD_DOWN){
			if ((boingo[2].ay < 0) || (boingo[2].dy < 224)){
				Erasing  = 2;			
				SPR_SetEraseData(0x8000,0,0,320,240);
				boingo[2].ay += 2;
				boingo[2].dy += 2;
				
				boingo[2].by += 2;
				boingo[2].cy += 2;
			}         
		}
		if (data & PAD_Z){
			Erasing  = 2;			
			SPR_SetEraseData(0x8000,0,0,320,240);
	
			if (abs(boingo[2].ax - boingo[2].bx) >= bg.width)
				direction ^= 1;
            if (!direction){		
				boingo[2].ax += 2;		
				boingo[2].dx += 2;
				boingo[2].bx -= 2;		
				boingo[2].cx -= 2;
			}else{
				boingo[2].ax -= 2;		
				boingo[2].dx -= 2;
				boingo[2].bx += 2;		
				boingo[2].cx += 2;		
			}
            if (boingo[2].ax < boingo[2].bx){    
			    boingo[2].ay += 2;
				boingo[2].by -= 2;		
				boingo[2].cy += 2;		
				boingo[2].dy -= 2;           
			}else{             
				boingo[2].ay -= 2;						
				boingo[2].by += 2;		
				boingo[2].cy -= 2;		
				boingo[2].dy += 2;
			}
		}
		
		switch(data){
			case PAD_A:
				Erasing  = 2;
				{
					int wdth,hth;
					
					wdth = boingo[2].cx - boingo[2].ax;
					hth = boingo[2].cy - boingo[2].ay;
					
					SPR_SetEraseData(0x8000,0,0,320,240);            
					boingo[2].bx = boingo[2].cx += wdth >> 5;
					boingo[2].cy = boingo[2].dy += hth >> 5;
					boingo[2].ax = boingo[2].dx -= wdth >> 5;
					boingo[2].ay = boingo[2].by -= hth >> 5;
				}
				break;
			case PAD_B:
				Erasing  = 2;			
				{
					int wdth,hth;
					
					wdth = boingo[2].cx - boingo[2].ax;
					hth = boingo[2].cy - boingo[2].ay;
					
					SPR_SetEraseData(0x8000,0,0,320,240);            
					boingo[2].bx = boingo[2].cx -= wdth >> 5;
					boingo[2].cy = boingo[2].dy -= hth >> 5;
					boingo[2].ax = boingo[2].dx += wdth >> 5;
					boingo[2].ay = boingo[2].by += hth >> 5;
				}
				break;
			default:
				break;
		}
		if ((boingo[2].ax < -1) || (boingo[2].cx > 320) || (boingo[2].ay < -1) || (boingo[2].cy > 240))
		    boingo[3].control = (JUMP_NEXT | ZOOM_NOPOINT | DIR_NOREV | FUNC_DISTORSP);
		else                   
			boingo[3].control = CTRL_END;	
	}
}
#endif /* INCLUDE_PAD */


void VbStart(void)
{   
#ifdef __GNUC__
asm("                 
	sts.l	macl,@-r15
	sts.l	mach,@-r15
");            
#endif
 	
 	vbs = 1;
               
#if INCLUDE_PAD
	SMPA_SwRead();
	
	doPad(SMPA_pad_edge1,SMPA_pad_data1, &sp);
	doPad(SMPA_pad_edge2,SMPA_pad_data2, &sp2);
	
#endif                
	SCL_VblankStart();   


 	vbs = 0;

#ifdef __GNUC__
asm("                 
	lds.l	@r15+,mach
	lds.l	@r15+,macl
");            
#endif
}

void swapSprites(void);

void VbEnd(void)
{ 
#ifdef __GNUC__
asm volatile("
	sts.l	macl,@-r15
	sts.l	mach,@-r15
");            
#endif
 	vbe = 1;
 
	SCL_VblankEnd();
	  
	if (!paused) loadAudio(playing);

 	vbe = 0;
#ifdef __GNUC__
asm volatile ("                
	lds.l	@r15+,mach
	lds.l	@r15+,macl
");            
#endif
}
#if VDP1SPRITE	 
#define PLAYER_INT_MASK INT_MSK_VBLK_IN|INT_MSK_VBLK_OUT|INT_MSK_SPR
#else
#define PLAYER_INT_MASK INT_MSK_VBLK_IN|INT_MSK_VBLK_OUT
#endif

void duckSetVBlank(void)
{   
	INT_ChgMsk(INT_MSK_NULL,INT_MSK_VBLK_IN|INT_MSK_VBLK_OUT|INT_MSK_SPR);
	
#if INCLUDE_PAD
	SMPA_SwInit();
#endif

	INT_SetScuFunc(INT_SCU_VBLK_IN,&VbStart);
	INT_SetScuFunc(INT_SCU_VBLK_OUT,&VbEnd);
	
#if VDP1SPRITE	 
	INT_SetScuFunc(INT_SCU_SPR,&swapSprites);
#endif
	
	INT_ChgMsk(INT_MSK_VBLK_IN|INT_MSK_VBLK_OUT|INT_MSK_SPR,INT_MSK_NULL);
}